// UruPython2.cpp : Defines the entry point for the console application.
//

/*
	This program is used to extract PYC files (Python code byte) stored in the .PAK file of Uru.  It can also
	recreated a PAK file from the PYC files.

	This program is not authorized/approved by neither Cyan or Ubisoft.  Don't ask them for any technical support.

	Anonymous54321

	March 26, 2004


  Version Histoty:

	1.0 => Original version

	1.0B => Added 'old style' format in the extract function

	1.2  => Added support for Myst V: End of Ages
					Added the 'fileList' option

*/

#include <stdafx.h>
#include <string.h>
#include <io.h>
#include <stdlib.h>
#include <time.h>
#include "aes.h"

int extractPython(char *sourceName, char *dirName, char *filelistName);
int pakPython(char *dirName, char *destName, int fileStyle, char *filelistName);

const int MaxNbrFile = 1000;
const int MaxLenFilename = 256;
int CreateListFromFile(char listOfFile[][MaxLenFilename], int maxNbrFile, char *filelistName, char *dirName,
											 size_t &totalSize, size_t &sizeHeader);
int CreateListFromDir(char listOfFile[][MaxLenFilename], int maxNbrFile, char *dirName,
											 size_t &totalSize, size_t &sizeHeader);

void decodeQuad(UINT *first, UINT *second);
void encodeQuad(UINT *first, UINT *second);

BOOL decodeUruStr(char *strEnc, char *realStr);
BOOL encodeUruStr(char *realStr, char *strEnc);

BOOL decodeUruStrOld(char *strEnc, char *realStr);

BOOL decodeEoAStr(char *strEnc, char *realStr);
BOOL encodeEoAStr(char *realStr, char *strEnc);

// Type of encrypted file
const int FileTypeEncrypt_None = 0;			// Not a valid encrypted file
const int FileTypeEncrypt_Uru = 1;			// Uru 'WhatDoYouSee' file
const int FileTypeEncrypt_EOA = 2;			// End of Ages 'AES' encrypted file
const int FileTypeEncrypt_UruOld = 3;		// Uru 'BriceIsSmart' file

int main(int argc, char* argv[])
{
	printf("UruPython 1.2 by Anonymous54321\r\n\r\n");

	if (argc >= 3) {
		if (argv[1][0] == 'e')
			return(extractPython(argv[2], argv[3], argv[4]));
		else if ((argv[1][0] == 'p') && (argv[1][1] == 'u'))
			return(pakPython(argv[2], argv[3], FileTypeEncrypt_Uru, argv[4]));
		else if ((argv[1][0] == 'p') && (argv[1][1] == 'e'))
			return(pakPython(argv[2], argv[3], FileTypeEncrypt_EOA, argv[4]));
	}

	printf("Usage:\r\n"
		     "  UruPython e source <destination directory> filelist  (extract PYC from PAK)\r\n"
				 "  UruPython pu directory destination filelist   (create PAK from PYC for Uru)\r\n"
				 "  UruPython pe directory destination filelist   (create PAK from PYC for EoA)\r\n\r\n"
				 "filelist is optionnal.  If provided with the 'e' command, it will be created\r\n"
				 "with the list of files extracted.  With the 'p' command, it will be used to\r\n"
				 "include only the listed file int the correct order.\r\n\r\n"
				 "Example: UruPython e python.pak UruPYCdir ListFile.txt\r\n"
				 "         UruPython pu UruPYCdir python.pak ListFile.txt\r\n"
				 "         UruPython pe EoAPYCdir python.pak ListFile.txt\r\n");
	
	return 0;
}


// Extract PYC from a PAK file
int extractPython(char *sourceName, char *dirName, char *filelistName)
{
	if (!sourceName) {
		printf("Invalid sourceName\r\n");
		return 0;
	}
	if (dirName) {
		// Make sure its a directory, not a file
		_finddata_t fileInfo;
		long resultFind = _findfirst(dirName, &fileInfo);
		if (resultFind < 0) {
			// Create the directory since it does not exist
			if (!CreateDirectory(dirName, NULL)) {
				printf("Unable to create destination directory\r\n");
				return 0;
			}
		} else {
			// Check attribute
			_findclose(resultFind);
			if ((fileInfo.attrib & _A_SUBDIR) != _A_SUBDIR) {
				printf("'destination' must be a directory\r\n");
				return 0;
			}
		}
	}

	// Try to read the file in an internal buffer
	FILE *srcHandle = fopen(sourceName, "rb");
	if (!srcHandle) {
		printf("Unable to read source file\r\n");
		return 0;
	}

	// Get file size
	if (fseek(srcHandle, 0, SEEK_END)) {
		printf("Error with fseek (1)\r\n");
		fclose(srcHandle);
		return 0;
	}
	size_t sourceSize = ftell(srcHandle);
	if (fseek(srcHandle, 0, SEEK_SET)) {
		printf("Error with fseek (2)\r\n");
		fclose(srcHandle);
		return 0;
	}
	if (sourceSize < 16) {
		printf("Source file too small\r\n");
		fclose(srcHandle);
		return 0;
	}

	// Read the whole file in a buffer
	char *buffer = (char *)malloc(sourceSize);
	if (!buffer) {
		printf("Out of memory (buffer)\r\n");
		fclose(srcHandle);
		return 0;
	}
	if (fread(buffer, 1, sourceSize, srcHandle) != sourceSize) {
		printf("Read short on source file (buffer)\r\n");
		free(buffer);
		fclose(srcHandle);
		return 0;
	}

	fclose(srcHandle);

	// Check for the End of Ages header
	size_t startOffset;
	int fileStyle = FileTypeEncrypt_None;
	if (*((int *)buffer) == 0x0d874288) {
		fileStyle = FileTypeEncrypt_EOA;
		startOffset = 0x08;
	} else {
		// Check for the 'whatdoyousee' header
		buffer[12] = 0;
		if (strcmp(buffer, "whatdoyousee") == 0) {
			fileStyle = FileTypeEncrypt_Uru;
			startOffset = 0x10;
		} else {
			// Check for the 'oldStyle' header
			if (strcmp(buffer, "BriceIsSmart") == 0) {
				fileStyle = FileTypeEncrypt_UruOld;
				startOffset = 0x10;
			} else {
				printf("Not a 'whatdoyousee' or 'BriceIsSmart' or 'End of Ages' file\r\n");
				free(buffer);
				return 0;
			}
		}
	}

	// Decode the file in memory
	size_t offsetData;
	if (fileStyle == FileTypeEncrypt_EOA) {
		// End of Ages encryption
		CCryptAES easObject;
		if (!easObject.decodeBuffer((uint8 *)(buffer + startOffset), sourceSize - startOffset)) {
			printf("Prolem while decoding the file with AES encryption\r\n");
			free(buffer);
			return 0;
		}
	} else {
		// Uru (old or new) encryption
		offsetData = startOffset;
		while (offsetData < sourceSize) {
			decodeQuad((UINT *)(buffer + offsetData), (UINT *)(buffer + offsetData + 4));
			offsetData += 8;
		}
	}

	// Creates the default python 2.2 header (adjust magic number)
	char pythonHeader[8];
	if (fileStyle == FileTypeEncrypt_EOA) {
		// EoA use Python 2.3.3
		pythonHeader[0] = 0x3b;
		pythonHeader[1] = char(0xf2);
	} else {
		// Uru use Python 2.2
		pythonHeader[0] = 0x2d;
		pythonHeader[1] = char(0xed);
	}

	pythonHeader[2] = 0x0d;
	pythonHeader[3] = 0x0a;
	*((long *)(pythonHeader + 4)) = time(NULL);

	// Create the filelist if needed
	FILE *outFileList = NULL;
	char fullName[1024];
	if (filelistName && (strlen(filelistName) > 0)) {
		sprintf(fullName, "%s\\%s", (dirName?dirName:"."), filelistName);
		outFileList = fopen(fullName, "wt");
		if (!outFileList) {
			printf("unable to create output file [%s]\r\n", fullName);
			free(buffer);
			return 0;
		}
	}

	// Extract file one after the other
	FILE *outHandle;
	char baseName[256];
	int nbrFile = *((int *)(buffer + startOffset));
	offsetData = startOffset + sizeof(int);
	size_t offsetPYC, sizePYC;
	for (int indexFile = 0; indexFile < nbrFile; indexFile++) {
		// Extract name of PY file
		if (fileStyle == FileTypeEncrypt_UruOld) {
			// Old Uru (BriceIsSmart)
			if (!decodeUruStrOld(buffer + offsetData, baseName)) {
				free(buffer);
				return 0;
			}
			offsetData += strlen(baseName) + 4;
		} else {
			// New Uru (WhatDoYouSee)
			if (fileStyle == FileTypeEncrypt_Uru) {
				if (!decodeUruStr(buffer + offsetData, baseName)) {
					free(buffer);
					return 0;
				}
				offsetData += strlen(baseName) + 2;
			} else {
				// End of Ages
				if (!decodeEoAStr(buffer + offsetData, baseName)) {
					free(buffer);
					return 0;
				}
				offsetData += strlen(baseName) + 2;
			}
		}
		
		// Extract offset of PYC data in buffer
		offsetPYC = *((size_t *)(buffer + offsetData)) + startOffset;
		offsetData += 4;
		if (offsetPYC > sourceSize) {
			printf("Offset too large for [%s]\r\n", baseName);
			free(buffer);
			return 0;
		}

		// Extract PYC size located at 'offsetPYC'
		sizePYC = *((size_t *)(buffer + offsetPYC));
		if (offsetPYC + sizePYC > sourceSize) {
			printf("Offset+size too large for [%s]\r\n", baseName);
			free(buffer);
			return 0;
		}

		// Create output file
		sprintf(fullName, "%s\\%sc", (dirName?dirName:"."), baseName);
		outHandle = fopen(fullName, "wb");
		if (!outHandle) {
			printf("unable to create output file [%s]\r\n", fullName);
			free(buffer);
			return 0;
		}

		// Write Python header
		if (fwrite(pythonHeader, 1, 8, outHandle) != 8) {
			printf("Error writing Python header for [%s]\r\n", fullName);
			fclose(outHandle);
			free(buffer);
			return 0;
		}
		
		// Write PYC data
		if (fwrite(buffer + offsetPYC + 4, 1, sizePYC, outHandle) != sizePYC) {
			printf("Error writing Python data for [%s]\r\n", fullName);
			fclose(outHandle);
			free(buffer);
			return 0;
		}

		if (fclose(outHandle) != 0) {
			printf("Error closing file [%s]\r\n", fullName);
			free(buffer);
			return 0;
		}

		// Add the name to the file list
		if (outFileList) {
			if (fprintf(outFileList, "%sc\r\n", baseName) < 0) {
				printf("Error writing the fileList\r\n");
				free(buffer);
				return 0;
			}
		}
	}

	// Close the file list if needed
	if (outFileList) {
		if (fclose(outFileList) != 0) {
			printf("Error closing the fileList\r\n");
			free(buffer);
			return 0;
		}
	}

	free(buffer);

	printf("Number of file created: %d\r\n", indexFile);

	return 1;
}


// Create a PAK from a list of PYC files
int pakPython(char *dirName, char *destName, int fileStyle, char *filelistName)
{
	if (!dirName || !destName) {
		printf("Invalid dirName or destName\r\n");
		return 0;
	}

	// Create the list of file to process (either all the PYC file or only the one listed in filelistName)
	char listOfFile[MaxNbrFile][MaxLenFilename];
	int nbrFile;
	char fullName[1024];
	size_t totalSize = 0;			// Total size not counting the whatdouyousee header (16 bytes)
	size_t sizeHeader = 4;		// Will be the size of the list of filename with offset (include at least the number of file)
	if (filelistName) {
		nbrFile = CreateListFromFile(listOfFile, MaxNbrFile, filelistName, dirName, totalSize, sizeHeader);
	} else {
		nbrFile = CreateListFromDir(listOfFile, MaxNbrFile, dirName, totalSize, sizeHeader);
	}
	if (nbrFile < 0) {
		printf("Error while creating the list of PYC file\r\n");
		return 0;
	}
	if (nbrFile < 1) {
		printf("No PYC file to store\r\n");
		return 0;
	}

	totalSize += sizeHeader;

	// Adjust parameter for the encryption
	size_t startOffset;
	int boundary;
	if (fileStyle == FileTypeEncrypt_EOA) {
		// End of Ages
		startOffset = 8;
		boundary = 16;
	} else {
		// Assume Uru
		startOffset = 16;
		boundary = 8;
	}

	// Adjust output size to be on an right bytes boundary (for the encryption) and add the size of the header
	size_t totalBuffer = totalSize;
	int diffBound = totalBuffer % boundary;
	if (diffBound)
		totalBuffer += (boundary - diffBound);
	totalBuffer += startOffset;

	// Creates a buffer to store all data in memory
	char *buffer = (char *)malloc(totalBuffer);
	if (!buffer) {
		printf("Out of memory\r\n");
		return 0;
	}

	// Store header information
	if (fileStyle == FileTypeEncrypt_EOA) {
		// End of Ages
		*((long *)(buffer + 0)) = 0x0d874288;
		*((long *)(buffer + 4)) = totalSize;
	} else {
		// Assume Uru
		strcpy(buffer, "whatdoyousee");
		*((long *)(buffer + 12)) = totalSize;
	}
	*((long *)(buffer + startOffset)) = nbrFile;

	// Now scan the file again and stored them into buffer
	char pattern[1024];
	long resultFind;
	_finddata_t fileInfo;
	int newNbrFile = 0;		// Just to double check the number of files
	size_t offsetHeader = startOffset + 4;
	size_t offsetData = offsetHeader + sizeHeader - 4;		// need to get rid of the +4 in offsetHeader which is also in sizeHeader

	FILE *srcHandle;
	char shortName[256];
	do {
		// Locate the file
		sprintf(pattern, "%s\\%s", dirName, listOfFile[newNbrFile]);
		resultFind = _findfirst(pattern, &fileInfo);
		if (resultFind < 0) {
			printf("What have you done with the PYC files?\r\n");
			free(buffer);
			return 0;
		}
		_findclose(resultFind);

		// Check that it is a normal file
		if ((fileInfo.attrib & (_A_HIDDEN | _A_SUBDIR | _A_SYSTEM)) == 0) {
			newNbrFile++;

			// Stored filename in header
			strcpy(shortName, fileInfo.name);
			shortName[strlen(shortName) - 1] = 0;			// Get rid of the C in PYC
			if (fileStyle == FileTypeEncrypt_EOA) {
				// End of Ages
				if (!encodeEoAStr(shortName, buffer + offsetHeader)) {
					free(buffer);
					return 0;
				}
			} else {
				// Assume Uru
				if (!encodeUruStr(shortName, buffer + offsetHeader)) {
					free(buffer);
					return 0;
				}
			}
			offsetHeader += strlen(shortName) + 2;
			
			// Store file offset in the header
			*((long *)(buffer + offsetHeader)) = offsetData - startOffset;			// The -startOffset is for the size of the header
			offsetHeader += 4;

			// Check for header overflow in the data section (the +startOffset if for the header)
			if (offsetHeader > (sizeHeader + startOffset)) {
				printf("Header buffer overflow in data buffer at file [%s]\r\n", fileInfo.name);
				free(buffer);
				return 0;
			}

			// Open source file
			sprintf(fullName, "%s\\%s", dirName, fileInfo.name);
			srcHandle = fopen(fullName, "rb");
			if (!srcHandle) {
				printf("Error opening file [%s] %d\r\n", fileInfo.name, errno);
				free(buffer);
				return 0;
			}

			// Jump over the magic number (keep timestamp but will be changed to file size)
			if (fseek(srcHandle, 4, SEEK_SET) != 0) {
				printf("Error with seek for file [%s]\r\n", fileInfo.name);
				fclose(srcHandle);
				free(buffer);
				return 0;
			}
			
			// Check for buffer overflow (-4 because I don't read magic number and -startOffset because of the header)
			if ((offsetData + fileInfo.size - 4 - startOffset) > totalSize) {
				printf("Data buffer overflow at file [%s]\r\n", fileInfo.name);
				fclose(srcHandle);
				free(buffer);
				return 0;
			}

			if (fread(buffer + offsetData, 1, fileInfo.size - 4, srcHandle) != (fileInfo.size - 4)) {
				printf("Short read for file [%s]\r\n", fileInfo.name);
				fclose(srcHandle);
				free(buffer);
				return 0;
			}
			fclose(srcHandle);

			// Change timestamp to file size
			*((long *)(buffer + offsetData)) = fileInfo.size - 4 - 4;		// Don`t count the magic number and the timestamp
			offsetData += fileInfo.size - 4;
		}
	} while (newNbrFile < nbrFile);

	// Make sure we have stored the correct number of file
	if (newNbrFile != nbrFile) {
		printf("Number of PYC file mismatch?\r\n");
		free(buffer);
		return 0;
	}
	if (offsetData != (totalSize + startOffset)) {
		printf("PYC file size mismatch?\r\n");
		free(buffer);
		return 0;
	}

	// Encrypt the buffer
	if (fileStyle == FileTypeEncrypt_EOA) {
		// End of Ages
		CCryptAES easObject;
		if (!easObject.encodeBuffer((uint8 *)(buffer + startOffset), totalBuffer - startOffset)) {
			printf("Prolem while encoding the file with AES encryption\r\n");
			free(buffer);
			return 0;
		}
	} else {
		// Assume Uru
		offsetData = startOffset;
		while (offsetData < totalBuffer) {
			encodeQuad((UINT *)(buffer + offsetData), (UINT *)(buffer + offsetData + 4));
			offsetData += 8;
		}
	}

	// Save buffer to disk
	FILE *destHandle = fopen(destName, "wb");
	if (!destHandle) {
		printf("Error creating output file?\r\n");
		free(buffer);
		return 0;
	}

	if (fwrite(buffer, 1, totalBuffer, destHandle) != totalBuffer) {
		printf("Error writing output file?\r\n");
		fclose(destHandle);
		free(buffer);
		return 0;
	}

	if (fclose(destHandle) != 0) {
		printf("Error closing output file?\r\n");
		free(buffer);
		return 0;
	}


	free(buffer);

	
	printf("Number of file stored: %d\r\n", nbrFile);

	return 1;
}


// Decode two UINT
void decodeQuad(UINT *first, UINT *second)
{
	UINT out1 = *first, out2 = *second, base = 0xc6ef3720;
	UINT temp1, temp2, crossRef[4];

	crossRef[0] = 0x6c0a5452;
	crossRef[1] = 0x03827d0f;
	crossRef[2] = 0x3a170b92;
	crossRef[3] = 0x16db7fc2;

	for (int index = 0x20; index > 0; index--) {
		// First word
		temp1 = temp2 = out1;
		temp1 >>= 5;
		temp2 <<= 4;
		temp1 ^= temp2;
		temp1 += out1;

		temp2 = base;
		temp2 >>= 0x0b;
		temp2 &= 0x03;

		temp2 = crossRef[temp2];
		temp2 += base;
		base += 0x61c88647;

		temp1 ^= temp2;
		out2 -= temp1;

		// Second word
		temp1 = temp2 = out2;
		temp1 >>= 5;
		temp2 <<= 4;
		temp1 ^= temp2;
		temp1 += out2;

		temp2 = base;
		temp2 &= 0x03;

		temp2 = crossRef[temp2];
		temp2 += base;

		temp1 ^= temp2;
		out1 -= temp1;
	}

	*first = out1;
	*second = out2;

}

// Encode two UINT
void encodeQuad(UINT *first, UINT *second)
{
	UINT out1 = *first, out2 = *second, base = 0;
	UINT temp1, temp2, crossRef[4];

	crossRef[0] = 0x6c0a5452;
	crossRef[1] = 0x03827d0f;
	crossRef[2] = 0x3a170b92;
	crossRef[3] = 0x16db7fc2;

	for (int index = 0x20; index > 0; index--) {
		// Second word
		temp1 = temp2 = out2;
		temp1 >>= 5;
		temp2 <<= 4;
		temp1 ^= temp2;
		temp1 += out2;

		temp2 = base;
		temp2 &= 0x03;

		temp2 = crossRef[temp2];
		temp2 += base;
		base -= 0x61c88647;

		temp1 ^= temp2;
		out1 += temp1;

		
		// First word
		temp1 = temp2 = out1;
		temp1 >>= 5;
		temp2 <<= 4;
		temp1 ^= temp2;
		temp1 += out1;

		temp2 = base;
		temp2 >>= 0x0b;
		temp2 &= 0x03;

		temp2 = crossRef[temp2];
		temp2 += base;

		temp1 ^= temp2;
		out2 += temp1;

	}

	*first = out1;
	*second = out2;
}

// This function decode a Uru string (new type which is inverted).  'strEnc' pointed to the encrypted string while
// 'realStr' should point to a buffer of at least 256 bytes (max len for uru string).  Returns FALSE in case of problem.
BOOL decodeUruStr(char *strEnc, char *realStr)
{
	if (!strEnc || !realStr) {
		printf("Bad parameter in decodeUruStr\r\n");
		return FALSE;
	}
	if (strEnc[1] != char(0xf0)) {
		printf("Not a Uru string (decodeUruStr)\r\n");
		return FALSE;
	}

	int length = strEnc[0];
	memcpy(realStr, strEnc + 2, length);
	realStr[length] = 0;
	for (int index = 0; index < length; index++)
		realStr[index] ^= 0xff;

	return TRUE;
}

// This function create a Uru string (new type which is inverted) based on 'realStr'.  The result is in 'strEnc'.
// Returns FALSE in case of problem.
BOOL encodeUruStr(char *realStr, char *strEnc)
{
	if (!strEnc || !realStr) {
		printf("Bad parameter in encodeUruStr\r\n");
		return FALSE;
	}

	int length = strlen(realStr);
	if (length > 255) {
		printf("String too long in encodeUruStr\r\n");
		return FALSE;
	}

	strEnc[0] = length & 0xff;
	strEnc[1] = char(0xf0);

	memcpy(strEnc + 2, realStr, length);
	for (int index = 0; index < length; index++)
		strEnc[index + 2] ^= 0xff;

	return TRUE;
}


// This function decode a Uru string (old style which is not inverted and string length on 4 bytes).  'strEnc' pointed
// to the encrypted string while 'realStr' should point to a buffer of at least 256 bytes (normally, it should be up to
// 65536 but I limit it to 256 in thid function). Returns FALSE in case of problem.
BOOL decodeUruStrOld(char *strEnc, char *realStr)
{
	if (!strEnc || !realStr) {
		printf("Bad parameter in decodeUruStr\r\n");
		return FALSE;
	}
	int length = *((int *)strEnc);
	if (length > 255) {
		printf("String too long (decodeUruStrOld)\r\n");
		return FALSE;
	}

	memcpy(realStr, strEnc + 4, length);
	realStr[length] = 0;

	return TRUE;
}


// This function decode an End of Ages string.  'strEnc' pointed to the encrypted string while
// 'realStr' should point to a buffer of at least 256 bytes (max len for EoA string).  Returns FALSE in case of problem.
BOOL decodeEoAStr(char *strEnc, char *realStr)
{
	if (!strEnc || !realStr) {
		printf("Bad parameter in decodeEoAStr\r\n");
		return FALSE;
	}

	int length = strEnc[0];
	memcpy(realStr, strEnc + 2, length);
	realStr[length] = 0;

	char *keyStr = "mystnerd";
	int curKeyChar = 0, keyLength = strlen(keyStr);
	for (int index = 0; index < length; index++) {
		realStr[index] ^= keyStr[curKeyChar];
		
		curKeyChar++;
		if (curKeyChar >= keyLength)
			curKeyChar = 0;
	}

	return TRUE;
}

// This function create an End of Ages string based on 'realStr'.  The result is in 'strEnc'.
// Returns FALSE in case of problem.
BOOL encodeEoAStr(char *realStr, char *strEnc)
{
	if (!strEnc || !realStr) {
		printf("Bad parameter in encodeEoAStr\r\n");
		return FALSE;
	}

	int length = strlen(realStr);
	if (length > 255) {
		printf("String too long in encodeEoAStr\r\n");
		return FALSE;
	}

	strEnc[0] = length & 0xff;
	strEnc[1] = char(0x00);

	memcpy(strEnc + 2, realStr, length);

	char *keyStr = "mystnerd";
	int curKeyChar = 0, keyLength = strlen(keyStr);
	for (int index = 0; index < length; index++) {
		strEnc[index + 2] ^= keyStr[curKeyChar];

		curKeyChar++;
		if (curKeyChar >= keyLength)
			curKeyChar = 0;
	}

	return TRUE;
}

// This function will create a valid list of PYC file based on the name listed in file filelistName.  It returns
// the number of file stored in 'listOfFile' but without going over maxNbrFile.  It returns -1 in case of error.
// This function will also compute the total size of all file and the size of the header.
int CreateListFromFile(char listOfFile[][MaxLenFilename], int maxNbrFile, char *filelistName, char *dirName, 
											 size_t &totalSize, size_t &sizeHeader)
{
	if (!filelistName || (strlen(filelistName) < 1))
		return -1;

	char fullName[1024];
	FILE *outFileList = NULL;
	if (filelistName && (strlen(filelistName) > 0)) {
		sprintf(fullName, "%s\\%s", (dirName?dirName:"."), filelistName);
		outFileList = fopen(fullName, "rt");
		if (!outFileList)
			return -1;
	} else
		return -1;

	totalSize = 0;			// Total size not counting the whatdouyousee header (16 bytes)
	sizeHeader = 4;		// Will be the size of the list of filename with offset (include at least the number of file)
	int nbrFile = 0;
	_finddata_t fileInfo;
	long resultFind;
	int isBadFile;
	char *posChar;
	while (fgets(listOfFile[nbrFile], MaxLenFilename, outFileList)) {
		// Get rid of return/linefeed
		posChar = strchr(listOfFile[nbrFile], '\r');
		if (posChar)
			*posChar = 0;

		posChar = strchr(listOfFile[nbrFile], '\n');
		if (posChar)
			*posChar = 0;

		// Validate that the file exist
		sprintf(fullName, "%s\\%s", (dirName?dirName:"."), listOfFile[nbrFile]);
		resultFind = _findfirst(fullName, &fileInfo);
		isBadFile = 1;
		if (resultFind >= 0) {
			// Check that it is a normal file
			if ((fileInfo.attrib & (_A_HIDDEN | _A_SUBDIR | _A_SYSTEM)) == 0) {

				// Will need space to store the name of the file minus the C of PYC.  The Uru string then add 2 bytes.  I then
				// need 4 bytes for the offset
				sizeHeader += strlen(fileInfo.name) - 1 + 2 + 4;

				// The file will be stored completely except for the 4 magic number (the timestamp will be replaced by file size)
				totalSize += fileInfo.size - 4;

				isBadFile = 0;
			}
		}

		_findclose(resultFind);

		if (isBadFile) {
			fclose(outFileList);
			return -1;
		}

		nbrFile++;
		if (nbrFile >= maxNbrFile) {
			fclose(outFileList);
			return -1;
		}
	}

	if (ferror(outFileList))
		return -1;

	if (fclose(outFileList) != 0)
		return -1;

	return nbrFile;
}


// This function will create a valid list of PYC file based on list of PYC file in directory 'dirName.  It returns
// the number of file stored in 'listOfFile' but without going over maxNbrFile.  It returns -1 in case of error.
// This function will also compute the total size of all file and the size of the header.
int CreateListFromDir(char listOfFile[][MaxLenFilename], int maxNbrFile, char *dirName, 
											 size_t &totalSize, size_t &sizeHeader)
{
	char pattern[1024];
	sprintf(pattern, "%s\\*.pyc", dirName);
	_finddata_t fileInfo;
	long resultFind = _findfirst(pattern, &fileInfo);
	int noMoreFile;
	int nbrFile = 0;
	if (resultFind >= 0) {
		do {
			// Check that it is a normal file
			if ((fileInfo.attrib & (_A_HIDDEN | _A_SUBDIR | _A_SYSTEM)) == 0) {
				strcpy(listOfFile[nbrFile], fileInfo.name);

				nbrFile++;

				// Will need space to store the name of the file minus the C of PYC.  The Uru string then add 2 bytes.  I then
				// need 4 bytes for the offset
				sizeHeader += strlen(fileInfo.name) - 1 + 2 + 4;

				// The file will be stored completely except for the 4 magic number (the timestamp will be replaced by file size)
				totalSize += fileInfo.size - 4;
			}

			noMoreFile = _findnext(resultFind, &fileInfo);
		} while (!noMoreFile);
		_findclose(resultFind);
	}
	
	return nbrFile;
}

